package samples.catchtherain;

import javafx.geometry.BoundingBox;
import javafx.scene.paint.Color;

import java.util.Random;

/*
 *  The application logic for the game "Catch the Rain" (not fully implemented)
 *  Nothing visual here
 */
public class CatchTheRain {

    private static final Random rand = new Random();   // Instance variables, as much finals as possible
    private final RainDrop[] drops;
    private final int width;
    private final int height;
    private final Bucket bucket;
    private int points = 0;

    public CatchTheRain(Bucket bucket, int nDrops, int width, int height) { //Constructor
        this.bucket = bucket;
        this.width = width;
        this.height = height;
        this.drops = new RainDrop[nDrops];
        for (int i = 0; i < drops.length; i++) {
            drops[i] = newRainDrop();
        }
    }

    // The game "loop" called by a timer about every 1/60 sec.
    public void update() {
        for (int i = 0; i < drops.length; i++) {
            if (drops[i] != null) {
                drops[i].move();
                boolean hit = hitBucket(drops[i]);
                if ( hit ) {
                    points++;
                }
                if (hitFloor(drops[i]) || hit ) {
                    drops[i] = newRainDrop(); // Just dump the old object, will be garbage collected
                }
            }
        }
        bucket.move();
    }

    // ---------- Collision detection -----------------

    private boolean hitBucket(RainDrop d) {
        boolean xRange = bucket.getX() < d.getX() && d.getMaxX() < bucket.getMaxX();
        return xRange && d.getY() > bucket.getY();
    }

    private boolean hitFloor(RainDrop d) {
        return d.getY() > height;
    }

    // ----------- Utility -------------------------

    private RainDrop newRainDrop() {
        Color c = Color.rgb(rand.nextInt(255), rand.nextInt(255), rand.nextInt(255));
        RainDrop drop = new RainDrop(rand.nextInt(width), 0, rand.nextInt(10) + 2, c);
        drop.setSpeed(rand.nextInt(3) + 2);
        return drop;
    }

    // -------- Used by GUI event handlers ------------------

    public void bucketLeft() {
        bucket.setSpeed(0 < bucket.getX() ? -2 : 0);
    }

    public void bucketRight() {
        bucket.setSpeed(bucket.getMaxX() < width ? 2 : 0);
    }

    public void bucketStop() {
        bucket.setSpeed(0);
    }

    // ------- Used by GUI to render -------------------

    public RainDrop[] getDrops() {
        return drops;
    }

    public Bucket getBucket() {
        return bucket;
    }

    public int getPoints() {
        return points;
    }
}
