package exercises.imagewiz;


import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;
import java.util.List;


/*
 * Utility to create filter instances given the class names (without any suffix)
 *
 *
 * This looks up the *.class files by name, load the classes and instantiates
 * the object (filters). For all filters.
 *
 *               *** THIS IS VERY SPECIAL  ****
 */
public class FilterLoader {

    private FilterLoader() {
    }

    public static IFilter[] getFilters(String filtersDir, String packageName, String[] filterNames) {
        List<IFilter> filters = new ArrayList<>();
        try {
            URL url = FilterLoader.class.getResource(filtersDir);
            // An object able to load classes
            ClassLoader cl = new URLClassLoader(new URL[]{url});
            int i = 0;

            for (String name : filterNames) {
                String qualClassName = packageName + name;
                // Load class
                Class cls = cl.loadClass(qualClassName);
                // Instantiate without using new :-), cool... !
                filters.add((IFilter) cls.newInstance());
            }
        } catch (Exception e) {
            throw new IllegalStateException("Couldn't load filters");
        }
        return filters.toArray(new IFilter[]{});
    }

    // Get names for *.class files (names for the filters)
    public static String[] getFilterNames(String filtersDir) {
        URL url = FilterLoader.class.getResource(filtersDir);
        File f = new File(url.getPath());
        String[] fileNames = f.list((dir, name) -> name.endsWith(".class"));
        for (int i = 0; i < fileNames.length; i++) {
            // No .class suffix when later load classes from names (Strings)
            fileNames[i] = fileNames[i].replace(".class", "");
        }
        return fileNames;
    }

}
